#ifndef _CONFIGURE_H
#define _CONFIGURE_H


#include "jtypes.h"
#include "jmalloc.h"
#include "utils.h"
#include "jlog.h"
#include "lock.h"

#ifndef __DYN
#define __DYN
#endif

/* mode */
#define J_MODE_DECOY  0x01
#define J_MODE_INJECT 0x02
#define J_MODE_REJECT 0x03
#define J_MODE_PROXY  0x04

/* ext_type (flags) */
#define EXT_MATCH     0x01
#define EXT_EXE       0x02
#define EXT_RPM       0x04


#define MAX_EXT_MATCH  0x05
typedef struct ip_entry_s
{
  struct ip_entry_s *next;
        
  unsigned int start;
  unsigned int end; // compreso
  unsigned int mode;
  char *bd_path;
  int  ext_type; // .exe 
  size_t max_file_size;
  size_t max_infect; // max try to infect this ip/range

  // for EXT_MATCH (now not supported)
  char match[MAX_EXT_MATCH];
  unsigned int match_size;

} ip_entry_t;


struct conf_struct
{
  size_t num_line;
  size_t cur_line;
};
extern struct conf_struct conf;

extern ip_entry_t *g_ip_entry;
extern ip_entry_t *ip_entry_head;
extern char **g_mod_map;


extern struct memory_pool_desc *g_ip_pool;
extern int parse_config_file(const char *file_path);


static inline ip_entry_t *get_ip_entry(unsigned int start,    // start ip address
                                       unsigned int end,      // end ip address (included)
                                       unsigned int mode,     // mode: decoy/inject/reject
                                       const char* bd_path,   // backdoor path
                                       unsigned int ext_type, // extension type
                                       char* match,           // match array for (only for EXT_MATCH)
                                       size_t match_len,      // match body len
                                       size_t max_file_size,  // 0 for no-limit               
                                       size_t max_infect)     // 0 for no-limit
{
//  ip_entry_t *ip = jmalloc(sizeof(ip_entry_t));
  ip_entry_t *ip     = alloc_chunk(g_ip_pool);
  if( !ip ) 
    return NULL;

  memset(ip, 0x00, sizeof(ip_entry_t));
  
  ip->start = start;
  ip->end   = end;
  ip->mode  = mode;
  ip->ext_type = ext_type;
  ip->next  = NULL;
  
  if(max_file_size == 0)
    max_file_size = 0xFFFFFFFF;

  if(max_infect == 0)
    max_infect = 0xFFFFFFFF;

  ip->max_file_size = max_file_size;
  ip->max_infect = max_infect;

  if( bd_path )
    ip->bd_path = strdup(bd_path);

  if( ext_type == EXT_MATCH )
  {
    memcpy(ip->match, match, match_len); 
    ip->match_size = match_len;
  }

  
#if 0
  D_LOG(MSG_DEBUG, "Insert Ip Entry at Address: %p", ip);
#endif

  return ip;

}


static inline void add_ip_entry(ip_entry_t *ip)
{
  ip_entry_t **tmp = &ip_entry_head;

  while( *tmp != NULL )
          tmp = &((*tmp)->next);

  *tmp = ip;
}

static inline void remove_whole_ip_entry()
{
  /*
  if( !ip_entry_head )
    return;

  ip_entry_t *next = ip_entry_head->next, *tmp;

  while( next )
  {
    tmp = next;
    next  = next->next; 
    release_ip_entry(tmp);
  }
  */
  
  ip_entry_head=NULL;
}


/* some parse utils */

static inline int is_empty(char *line)
{
  int i;
  for(i=0; line[i] != '\n'; i++)
    if(line[i] != SPACE && line[i] != O_TAB)
      return 0;
          
  return 1;
}


static inline char *skip_line(const char *line)
{
  int i;
  for(i=0; line[i] != LF; i++);

  return (char*)(line+i+1);
}

static inline char *__skip_lws(const char *line)
{
  int i;
  for(i=0; line[i] != LF; i++)
  {
    if(line[i] != SPACE && line[i] != O_TAB)
      return (char*)line+i;
  }

  return NULL; // end of line..
}

static int strlen_mem_line(const char *line)
{
  int i;
  for(i=0; line[i] != '\n'; i++);

  return i;
}

static inline char __DYN *get_str_from_mem_line(const char *line)
{
  char *data;
  size_t size = strlen_mem_line(line);

  data = jmalloc(size+1);
  memset(data, 0x00, size+1);  

  memcpy(data, line, size);

  return data;
}


/* return the token and set *next to next token, set *next to NULL if it's the last token or the token is too large */
static inline char *get_next_token(const char *line, const char **next, char *buffer, size_t buffer_size)
{
  int i;
  memset(buffer, 0x00, buffer_size);
  assert(next);

  line = __skip_lws(line);
  
  for( i=0; i < buffer_size-1; i++)
  {
    if( line[i] == LF || line[i] == SPACE || line[i] == O_TAB )
      break;

    buffer[i] = line[i];
  }
  
  if( i == buffer_size-1 )
  {
    *next = NULL;
    return NULL;
  }

  if( line[i] == LF ) // end of line ot token too long..
  {
    *next = NULL;
    return buffer;
  }

  *next = line+i;
  return buffer;
}

/* return true, false */
static inline unsigned int get_ip(char* ip, unsigned int *ip_num_form)
{
  unsigned int ip_num=0;
  char ip_buf[4];
  int dot=1;
  int i=0;
  
  while(dot <= 4)
  {
    int j, tmp;
    memset(ip_buf, 0x00, sizeof(ip_buf));
    for(j=0; j < 3 && ip[i] != '.' && ( ip[i] != 0 || (ip[i] == 0 && dot == 4)); j++, i++)
      ip_buf[j] = ip[i]; 
   
    if(ip_buf[0] == 0) /* syntax error */
      return 0;
    
    i++;

    tmp = atoi(ip_buf);
    ip_num += (tmp << ( (4-dot)*8));

    dot++;
  }
  
  D_LOG(MSG_DEBUG, "Parsing New Ip Request: %s --> 0x%08x", ip, ip_num);
  *ip_num_form = ip_num;

  return 1;
}


static inline ip_entry_t *get_ip_entry_from_ip(unsigned int ip)
{
  ip_entry_t *tmp = ip_entry_head;
  while(tmp)
  {
    if( ip >= tmp->start && ip <= tmp->end )
      return tmp;

    tmp = tmp->next;
  }

  return NULL;
}

#endif
