#ifndef DATA_FRAGMENT_H
#define DATA_FRAGMENT_H

#include <string.h>
#include "jmalloc.h"

/* type */
#define DF_HEADER 0x01 /* a header line terminated by /r/n */
#define DF_RAW    0x02 /* raw data */
#define DF_CHUNK  0x03 /* raw data terminated by /r/n without traling chunk size */

/* flags */
#define DF_F_EOF               0x01
#define DF_F_ON_CLOSE          0x02
#define DF_F_ANCILLARY_HEADER  0x04

typedef struct data_fragment_s
{
  char *data;
  struct data_fragment_s *ancillary;
  size_t size;
  int type;
  int flags;
  
} data_fragment_t;

typedef data_fragment_t df_t;



static inline df_t *alloc_df(char *data, size_t size, int type, int flags)
{
  df_t *df = jmalloc(sizeof(df_t));
  if( !df )
    return NULL;

  memset(df, 0x00, sizeof(df_t));

  df->data  = data;
  df->size  = size;
  df->type  = type;
  df->flags = flags;

  return df;
}


static inline df_t *alloc_df_zero(void)
{
  return alloc_df(NULL, 0, 0, 0);
}

static inline void free_df(df_t *df)
{
  assert( df );
  if( df->ancillary )
    free_df(df->ancillary);

  if( df->data )
    jfree(df->data);

  jfree(df);
}

static inline df_t *alloc_empty_line_df()
{
  char *data = jmalloc(3);
  strcpy(data, "\r\n");
  df_t *df = alloc_df(data, 2, DF_HEADER, 0);

  return df;
}

static inline df_t *alloc_content_length_df(size_t size)
{
  char *buffer = jmalloc(64);
  memset(buffer, 0x00, 64);

  snprintf(buffer, 64, "Content-Length: %d\r\n", size);

  return alloc_df(buffer, strlen(buffer), DF_HEADER, 0);
}

#endif
