#ifndef J_HOOK_H
#define J_HOOK_H

#include "df.h"
#include "configure.h"

#define MAX_HOOK 16

#define INJECT_QUERY_CAN_HOOK          0x01

/* content info flags (set by hook function) 
 * NOTE: when a hook choise to process the input stream DONT call other hook handler...
 */
#define INJECT_HOOK_DATA_PRESENT        0x01  // there is new inject data
#define INJECT_HOOK_DATA_BUFFERED       0x02  // don't send nothing .. the data is bufferd by the hook (df_t is NULLIFIED)
#define INJECT_HOOK_UNMODIFIED          0x04  // the data was not modified at all
#define INJECT_HOOK_ERROR               0x08  // some error occurs..
#define INJECT_HOOK_FATAL               0x10  // some fatal error occurs.. the session must be closed.. and the error logged..

struct client_request;

/* function calling inject hook must free provided buffer if needed.. */
typedef int (*filter_hook)( df_t **new_df,                   // data fragment buffer
                            int query,                       // query type
                            int *to_do,                      // what the hook do
                            struct client_request *request,  // ptr to current client request (only read.. )
                            ip_entry_t *ip);                 // ptr to ip_entry_t structure


struct hook_desc
{
  filter_hook hook;   // hook function
  char desc[128];     // hook description
  unsigned int prio;  // hook priority
  int (*init)(ip_entry_t *entry); // hook init function  (return -1 on error, 0 if not registered > 1 if ok)
};


/* hook facility */
struct hook_filter_chain
{
  unsigned int index;
  struct hook_desc *array[MAX_HOOK];
};

extern struct hook_filter_chain g_client_filter_chain;
extern struct hook_filter_chain g_server_filter_chain;

extern ip_entry_t  *init_hook_engine(char *ip);

/* return the number of hook chain that have modified the packet */
extern int call_hook_chain(struct hook_filter_chain *hook_chain, df_t **orig, struct client_request *req, ip_entry_t *ip);

#endif
