#ifndef JSELECT_H
#define JSELECT_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include "jtypes.h"
#include "jmalloc.h"
#include "jlog.h"


//#define DEFAULT_SET_ENTRY_NUM 10


typedef struct j_socket_set_s
{
  j_socket **j_socket_array;
  /* number of obj max in the set */
  int num;
  /* current index of last obj in the set */
  int cur;
  /* current index of last obj retrieved */ 
  int ptr;

  /* last deleted index item */ 
  int del;

} j_socket_set_t;

static inline void rewind_j_socket_set(j_socket_set_t *set)
{
  set->ptr = 0;
}

static inline j_socket* get_next_j_socket(j_socket_set_t *set)
{
  j_socket *jsock;
  
  if( set->ptr > set->cur )
    return NULL;

  /* jump away deleted items */
  do {

    if( set->ptr > set->cur )
      return NULL;
    
    jsock = set->j_socket_array[set->ptr];
    set->ptr++;

  } while(jsock == NULL);

  return jsock;
}

static inline j_socket_set_t* get_j_socket_set(size_t start_size)
{
  j_socket_set_t *set = jmalloc(sizeof(j_socket_set_t));
  if( !set )
    return NULL;

  j_socket** array = jmalloc(sizeof(void*)*start_size);
  if( !array )
    return NULL;


  memset(set, 0x00, sizeof(j_socket_set_t));
  memset(array, 0x00, sizeof(void*)*start_size);

  set->num = start_size;
  set->cur = set->del = -1;
  set->ptr =  0;
  set->j_socket_array = array;

  //D_LOG(MSG_DEBUG, "Allocation of new set: num=%d", set->num);
  return set;
}

static inline void release_j_socket_set(j_socket_set_t *set)
{
  assert( set );
  assert( set->j_socket_array );
  
  jfree(set->j_socket_array);
  jfree(set);
}


static inline int add_j_socket_to_set(j_socket_set_t *set, j_socket *sock)
{
  /* TODO: add realloc of the set */

  assert( !(set->cur+1 > set->num)  );  

  if(set->del != -1)
  {
    set->j_socket_array[set->del] = sock;
    set->del = -1;
    return 1;
  }

  /* try to realloc */
  if( set->cur+1 == set->num)
  {
    j_socket** new_array = jmalloc( (set->num * 2) * sizeof(void*) );
    if( !new_array )
      return 0;

    memset(new_array, 0x00, sizeof(void*) * set->num * 2);  

    /* copy the new array */
    memcpy(new_array, set->j_socket_array, set->num*sizeof(void*)); 
    jfree(set->j_socket_array);
    
    set->j_socket_array = new_array;
    set->num = set->num*2;
    fprintf(stderr, "[*] Reallocation: num=%d\n", set->num);
  }

  set->cur++;
  set->j_socket_array[set->cur] = sock; 
  return 1;
}

static inline int del_j_socket_from_set(j_socket_set_t *set, j_socket *sock)
{
  size_t num = set->cur + 1,i;
  for(i=0; i<num; i++)
  {
    if( set->j_socket_array[i] == sock )
    {
      set->j_socket_array[i] = NULL;
      set->del = i;
      return 1;
    }
  }

  assert(0);
  return 0;
}


static inline void reset_j_socket_set(j_socket_set_t *set)
{
  set->cur = set->del = -1;
  set->ptr = 0;
  memset(set->j_socket_array, 0x00, sizeof(void*) * set->num);
}

/* worste-performance routine.. */
static inline int  is_j_socket_set(j_socket_set_t *set, j_socket *jsock)
{
  assert( jsock );
  
  size_t num = set->cur + 1,i;
  for(i=0; i<num; i++)
  {
    if(set->j_socket_array[i] == jsock)
      return 1;
  }
   
  return 0;
}

static inline j_socket_set_t* duplicate_set(j_socket_set_t* set)
{
  j_socket *tmp;
  j_socket_set_t *new_set = get_j_socket_set(set->num);


  rewind_j_socket_set(set);
  /* this does not count hole .. (get_next.. skeep them) */
  while( (tmp = get_next_j_socket(set)) )
  {
    if( !add_j_socket_to_set(new_set, tmp) )
    {
      release_j_socket_set(new_set);
      return NULL;
    }
     
  }
  return new_set;
}


/* I/O Multiplexing interface */
int js_select(j_socket_set_t *set, j_socket_set_t **new_set, int sec);

#endif
