#ifndef J_SOCKEK_H_
#define J_SOCKEK_H_

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include "jtypes.h"


/* jsocket opaque obj */
typedef struct j_socket_s
{
  void*   p_data;
  void*   session_data; /* can be used to bind session/socket */
  int     (*read)    (void* p_data, char *buffer, size_t len);
  int     (*write)   (void* p_data, const char *c_buffer, size_t len);
  int     (*close)   (void* p_data);
  int     (*connect) (void* p_data, const struct sockaddr_in *peer, int opt);
  int     (*listen)  (void* p_data);
  int     (*bind)    (void* p_data, const struct sockaddr_in *peer);
  int     (*getname) (void* p_data, struct sockaddr_in *peer);
  os_sock (*accept)  (void* p_data);
  os_sock (*getsock) (void* p_data);

} j_socket;



/* jsocket operation
 * basical socket manupulation function interface
 * virtual js_init() function is socket dependent function and we can't 
 * use a common interface: we can divide it into js_init_sock_tcp and
 * js_init_new_sock_tcp(), for ssl use like: ssl_js_init.. 
 * also ok for destroy..
 */

static inline void* js_session_read(j_socket *jsock)
{
  return jsock->session_data;
}

static inline void js_session_write(j_socket *jsock, void *session)
{
  jsock->session_data = session; 
}


static inline int js_read(j_socket* jsock, char* buffer, size_t len)
{
  return jsock->read(jsock->p_data, buffer, len);
}

static inline int js_write(j_socket* jsock, const char* c_buffer, size_t len)
{
  return jsock->write(jsock->p_data, c_buffer, len);
}

static inline int js_close(j_socket* jsock)
{
  return jsock->close(jsock->p_data); 
}

static inline int js_connect(j_socket* jsock, const struct sockaddr_in* peer, int opt)
{
  return jsock->connect(jsock->p_data, peer, opt);
}

static inline int js_bind(j_socket* jsock, const struct sockaddr_in* peer)
{
  return jsock->bind(jsock->p_data, peer);
}

/* use default backlog */
static inline int js_listen(j_socket* jsock)
{
  return jsock->listen(jsock->p_data);
}

static inline int js_getname(j_socket* jsock, struct sockaddr_in* peer)
{
  return jsock->getname(jsock->p_data, peer);
}

static inline os_sock js_getsock(j_socket* jsock)
{
  return jsock->getsock(jsock->p_data);
}

static inline os_sock js_accept(j_socket* jsock)
{
  return jsock->accept(jsock->p_data);
}





/**********************************IMPL ********************************/
/* simple sock  layer wrapper: op_socket.c */

j_socket* js_init_new_sock_tcp();
j_socket* js_init_sock_tcp(os_sock sock);
void      js_destroy_sock_tcp(j_socket*);

#endif
