#ifndef __LOCK_H
#define __LOCK_H

#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include "jlog.h"


// !!! those are TRUSTED functions, don't call them with some some sort of non validated user input!


/* return 1 when the lock is get, 0 when there is some error on getting it (busy wait) */
static inline int get_lock(const char *lock)
{
  int ret=-1;
  int err = EEXIST;
 
  while( (ret < 0)  &&  (err == EEXIST) )
  { 
    ret = mkdir(lock,  S_IRUSR| S_IWUSR); 
    
    err = errno;
    if( err != EEXIST )
    {
      D_LOG(MSG_ERROR, "Unable to get Lock: %s", lock);
      return 0;
    }
    
    usleep(200);
  }

  D_LOG(MSG_DEBUG, "Lock Aqcuired on: %s", lock); 
  return 0;
}


static inline void release_lock(const char *lock)
{
  {
    struct stat s;
    int ret = stat(lock, &s);
    assert(ret >= 0);
  }


  unlink(lock);
  return;
}


struct memory_pool_desc
{
  size_t chunk_num;
  size_t chunk_size;
  size_t chunk_max;
  char pool[0]; // gcc trick
};

#define PAGE_SIZE 0x1000
#define ALIGN_SIZE(size) (((size)+PAGE_SIZE) & ~(PAGE_SIZE-1))
static inline struct memory_pool_desc* create_mapping_pool(const char *tmp_file, size_t chunk_size, size_t chunk_max)
{
  struct memory_pool_desc desc;
  desc.chunk_num  = 0;
  desc.chunk_size = chunk_size; 
  desc.chunk_max  = chunk_max;

  int fd = open(tmp_file, O_RDWR | O_CREAT | O_TRUNC,  S_IRWXU);
  if( fd == -1)
    return NULL;
  
  size_t mmap_size =  ALIGN_SIZE( (chunk_max*chunk_size) + sizeof(struct memory_pool_desc) );
  lseek(fd, mmap_size, SEEK_SET);
  write(fd, "MARKER", 6);
  sync();
  
  D_LOG(MSG_DEBUG, "Mmap; size: 0x%x", mmap_size);
  void* addr = mmap((void*)0, mmap_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0); 
  if( MAP_FAILED == addr )
  {
    perror("mmap");
    return NULL;          
  }
  memset(addr, 0x00, mmap_size);
  memcpy(addr, &desc, sizeof(struct memory_pool_desc));

  return addr;
}

static inline void *alloc_chunk(struct memory_pool_desc *desc)
{
  if( desc->chunk_num == desc->chunk_max )
    return NULL;

  char *chunk = desc->pool + (desc->chunk_num * desc->chunk_size);
  desc->chunk_num++;

  return (void*)chunk;
}


#endif
