#ifndef UTILS_H
#define UTILS_H

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>
#include "jtypes.h"
#include "jmalloc.h"


#define __DYN // null macro identify function return dynamic allocated memory pointer ..

/* return NULL if error.. or the proper struct ptr (implicit PF_INET familty) */
static inline struct sockaddr_in* get_peer(const char* cp, unsigned short port)
{
  struct sockaddr_in *peer = jmalloc(sizeof(struct sockaddr_in)); 
  if( !peer )
    return NULL;
  
  peer->sin_family = PF_INET;
  peer->sin_port   = htons(port);
  if( !inet_aton(cp, &(peer->sin_addr)) )
  {
    jfree(peer);
    return NULL;
  } 

  return peer;
}


static inline void release_peer(struct sockaddr_in *peer)
{
  assert(peer);
  jfree(peer);
}


/* line (string+\n\r without 0) manupulation function */ 
static inline int is_lws(char a)
{
  if( a == SPACE || a == O_TAB )
    return 1;
  else
    return 0;
}



/* return NULL if len bytes of line are lwp: elsewher the first byte non lwp */
static inline LINE *skip_lws(const LINE *line, size_t len, size_t *rest)
{
  size_t i;
  for(i=0; i < len; i++)
  {
    if( !is_lws(*(line + i))  )
    {
      if( rest )
        *rest = len - i;

      return (LINE*)line + i;
    }
  }

  if ( rest )
    *rest = 0;

  return NULL;
}

/* assumed terminating with CR LF */
static inline size_t line_strlen(const LINE *line)
{
  size_t i;
  for(i=0; line[i] != 0x0D; i++);

  return i;
}       


static inline char __DYN *line_get_str(const LINE *line)
{
  size_t size = line_strlen(line);
  char *new_str = jmalloc(size+1);

  memset(new_str, 0x00, size+1);
  memcpy(new_str, line, size);

  return new_str;
}


static inline LINE *line_strmem(const LINE *line, const char* token, size_t len, size_t *rest)
{
  size_t i, token_len = strlen(token);
  if( token_len > len )
  {
    if( rest )
      *rest = 0;
    return NULL;
  }
  
  for(i=0; i <= len - token_len; i++)
  {
    if(!memcmp(line+i, token, token_len))
    {
      if( rest )
        *rest = len - i;
      
      return (LINE *)line+i;
    }
  }

  if( rest )
    *rest = 0;
  return NULL;
}


static inline int memcmp_nocase(const char* line1, const char* line2, size_t size)
{
  int i;
  for(i=0; i < size; i++)
  {
    if(tolower(line1[i]) != tolower(line2[i]))
      return 1;
  }

  return 0;
}

static inline LINE *line_strmem_nocase(const LINE* line, const char* token, size_t *rest)
{
  size_t token_len = strlen(token), line_len = line_strlen(line), i; 
  if( token_len > line_len )
  {
    if( rest )
      *rest = 0;
    return NULL;
  }

  for(i=0; i <= line_len - token_len; i++)
  {
    if(!memcmp_nocase(line+i, token, token_len))
    {
      if( rest )
        *rest = line_len - i;
      
      return (LINE *)line+i;
    }
  }

  if( rest )
    *rest = 0;
  return NULL;
  
}


static inline LINE* line_strmem_nocase_start(const LINE* line, const char* token)
{
  size_t size = strlen(token), line_size = line_strlen(line);
  if( line_size < size )
    return NULL;

    if(!memcmp_nocase(line, token, size))
      return (LINE*)line;
    else
      return NULL;
}

static inline char* __DYN line_get_next_tok(const LINE* line, size_t idx, size_t *rest)
{
  size_t size = line_strlen(line),i;
  const LINE *tmp = line + idx;

  if(idx >= size)
    return NULL;

  char *str = jmalloc(size+1);
  memset(str, 0x00, size+1);

  LINE *new_line = skip_lws(tmp, size-idx, NULL);
  idx += (new_line - tmp);

  for(i=0; new_line[i] != SPACE && new_line[i] != O_TAB && new_line[i] != CR; i++, idx++)
    str[i] = new_line[i];

  if( rest )
    *rest = idx;
  
  return str;
}


static inline int line_atoi(const LINE* line)
{
  char *str = line_get_str(line);
  int num = atoi(str);   
  jfree(str);

  return num;
}


static inline int line_strtol(const LINE* line)
{
  char *str = line_get_str(line);
  int num = strtol(line, NULL, 16);
  jfree(str);

  return num;
}

static inline LINE __DYN *line_get_chunk_size(size_t size, size_t *new_size)
{
  char *line = jmalloc(16);
  if( !line )
    return NULL;

  memset(line, 0x00, 16);
  snprintf(line, 16, "%x\r\n", size);

  if( new_size )
    *new_size = line_strlen(line) + 2;

  return line;
}



static inline size_t get_file_size(int fd)
{
  struct stat s;
  int ret;

  ret = fstat(fd, &s);

  if(ret < 0)
    return ret;

  return s.st_size;
}


#define XOR_MASK 0x26

static inline void compress_file(FILE *source, FILE *dest)
{
  unsigned int byte, num_zero;
  while ( (byte=fgetc(source)) != EOF) {
    if (byte==0) {
      fputc((unsigned char)byte ^ XOR_MASK, dest);
      for (num_zero=1; num_zero<255 && (byte=fgetc(source))==0; num_zero++);
      fputc((unsigned char)num_zero-1, dest);

      if (num_zero == 255)
        continue;
    }
    if (byte!=EOF)
      fputc((unsigned char)byte^XOR_MASK, dest);
  }

  fflush(dest);
  rewind(source);
}

/* copy file and rewind source */
static inline void copy_file(FILE *source, FILE *dest)
{
  int ret;
  while( (ret = fgetc(source)) != EOF )
    fputc((unsigned char)ret, dest);

  fflush(dest);
  rewind(source);
}

static inline int is_exe_payload(const char* data, size_t len)
{
  if( len < 2)
    return 0;

  if(data[0] == 'M' && data[1] == 'Z')
    return 1;

  return 0;
}

static inline char *convert_ip(unsigned int ip_little_endian)
{
  struct in_addr addr;
  addr.s_addr = htonl(ip_little_endian);
  return strdup(inet_ntoa(addr));
}

#endif
