#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/signal.h>
#include "jtypes.h"
#include "jsocket.h"
#include "jselect.h"
#include "jproxy.h"
#include "jlog.h"
#include "utils.h"
#include "configure.h"


#define DEFAULT_CONFIG_FILE "/etc/jproxy.conf"
#define DEFAULT_BIND_PORT   (80)
#define DEFAULT_BIND_ADDR   "0.0.0.0"
#define DEFAULT_INFECT_FILE "/var/log/jproxy_infect.log"

#define FATAL_SYS_IF(code, contest) { if(code < 0) { perror(contest); abort(); } }
#define FATAL_J_IF(code, contest) { if( !code ) { fprintf(stderr, "Fatal: %s\n", contest); abort(); } }


/* global vars */
static const char *g_config_file = DEFAULT_CONFIG_FILE;
static unsigned short g_bind_port   = DEFAULT_BIND_PORT;
static const char *g_bind_addr = DEFAULT_BIND_ADDR; 
FILE *infect_file=NULL;

int do_child(j_socket *jsock, unsigned int mode)
{
  int ret;
  proxy_session_t *session = init_proxy_session(jsock);  // TODO: make mode before init.. (useless connection active with decoy mode.. :(
  if( !session )
  {
    js_close(jsock);
    js_destroy_sock_tcp(jsock);
    D_LOG(MSG_ERROR, "Unable To Inizialize The Session");
    return 0;
  }
  ret = stream_proxy_session(session, mode);
  return ret;
}


void parse_args(int argc, char *argv[])
{
  int ret;
  while( (ret = getopt(argc, argv, "b:p:c:g:hV")) != -1)
  {
    switch(ret)
    {
      case 'c':
        g_config_file = strdup(optarg);
        break;

      case 'p':
        g_bind_port = atoi(optarg);
        break;

      case 'b':
        g_bind_addr = strdup(optarg);
        break;

      case 'h':
        fprintf(stderr, "Usage:  inject_proxy [-b bind_address] [-p bind_port] [-c config file]\n"
                        "        inject_proxy [-V]\n"
                        "        inject_proxy [-h]\n"
                        "\tDefault Value: bind_port=80\n"
                        "\tDefault Value: bind_addr=0.0.0.0\n"
                        "\tDefault Value: config_file=/etc/jproxy.conf\n"); 
        exit(0);
        break;

      case 'V':
        fprintf(stderr, "inject_proxy Ver. 0.1\n");
        exit(0);
        break;

      default:
       D_LOG(MSG_ERROR, "Unsupported Option: %c: (optarg? %s)", ret, optarg); 
       exit(1);
    }
  } 

  D_LOG(MSG_INFO, "Using Config File: %s", g_config_file);
  D_LOG(MSG_INFO, "Using Bind Port: %d", g_bind_port);
  D_LOG(MSG_INFO, "Using Bind Addr: %s", g_bind_addr);
}

int main(int argc, char *argv[])
{
  int ret, on = 1;
  struct sockaddr_in *peer_bind;

  parse_args(argc, argv);
  
 
  if( !parse_config_file(g_config_file))
    return 1;

  if(!infect_file_log)
    infect_file_log = DEFAULT_INFECT_FILE;

  infect_file = fopen(infect_file_log, "w");
  if( !infect_file )
  {
    D_LOG(MSG_ERROR, "Unable To Open Infection File Log: %s", infect_file_log);
    return 1;
  }

  j_socket* bind_proxy = js_init_new_sock_tcp();
    FATAL_J_IF(bind_proxy, "js_init_sock_tcp()"); 

  on = setsockopt(js_getsock(bind_proxy), SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
    FATAL_SYS_IF(on, "setsockopt()");


  peer_bind = get_peer(g_bind_addr, g_bind_port);  
  ret = js_bind(bind_proxy, peer_bind);
    FATAL_SYS_IF(ret, "js_bind()"); 
    
  release_peer(peer_bind);


  ret = js_listen(bind_proxy);
    FATAL_SYS_IF(ret, "js_listen()"); 


  signal(SIGPIPE, SIG_IGN);
    
  while(1)
  {
    ip_entry_t *ip = NULL;
    int ret;
    pid_t pid;
    struct sockaddr_in peer_client;
    j_socket *sock_client;
    os_sock real_sock;

    D_LOG(MSG_DEBUG, "Accepting New Connection:...");
    real_sock = js_accept(bind_proxy);  
      FATAL_SYS_IF(real_sock, "js_accept()"); 
    
    
    pid = fork();
      FATAL_SYS_IF(pid, "fork()");
    
    if(pid == 0) // child
    {
      int mode;
      ret = 0;
      sock_client = js_init_sock_tcp(real_sock); 
        FATAL_J_IF(sock_client, "js_init_sock_tcp()");

      js_close(bind_proxy);
      js_destroy_sock_tcp(bind_proxy);
 
      js_getname(sock_client, &peer_client); 

      D_LOG(MSG_DEBUG, "New Socket Accepted: %s:%d", inet_ntoa(peer_client.sin_addr), ntohs(peer_client.sin_port));

      ip = init_hook_engine(inet_ntoa(peer_client.sin_addr));
      if( !ip )
        mode = J_MODE_PROXY;
      else
        mode = ip->mode;
     
      g_ip_entry = ip;
      
      ret = do_child(sock_client, mode);
      exit(ret);  // exit
    }
    else        // parent
    {
      close(real_sock);  // close client socket
      ret = 1;
      while(ret>0)  
      {
        ret = waitpid(-1, NULL, WNOHANG); // wait prevs child dead...
        if(ret < 0)
          FATAL_SYS_IF(ret, "wait()");
      }
    } 

  } /* while(1) */

  return 0; 
}
