#include "jproxy.h"
#include "jhook.h"
#include "utils.h"
#include "line_buffer.h"
#include "jlog.h"
#include "df.h"

#define __F_MAY_HAVE_BODY 0x1
#define __F_ZERO_BODY     0x2
#define __UNKNOWN_LEN_MARKER 0xFFFFFFFFUL

static inline LINE* is_header_type(const LINE* line, const char* header, size_t *rest)
{
  LINE *new_line=NULL;
  size_t  sub_len;
  
  if( (new_line = line_strmem_nocase(line, header, &sub_len)) )
  {
    new_line += strlen(header);
    return skip_lws(new_line, sub_len, rest);
    
  }
  
  if( rest )
    *rest = 0;

  return NULL;
}

/* return > 0 if the request was modified */
int request_header_modifier(proxy_session_t *session, df_t *df, int peer)
{
  size_t rest;
  LINE *line;
  char *token;
  assert(df->type == DF_HEADER && (peer == PEER_CLIENT || peer == PEER_SERVER) );
  
  struct client_request *req = get_cur_request(session, peer);

  if( (line = line_strmem_nocase_start(df->data, "GET")) )
  {
    size_t idx=0;
    while( (token = line_get_next_tok(line, idx, &idx)) != NULL)
    {
      if(strstr(token, ".exe"))
      {
        D_LOG(MSG_INFO, "*.exe found in the GET request: setting MSDOS type");
        req->info_client |= REQUEST_TYPE_MSDOS;
      }        
      jfree(token);
    } 
  } 
  else if( (line = is_header_type(df->data, "Content-Disposition:", &rest)) )
  {
    if(line_strmem_nocase(line, "exe", &rest))
    {
      D_LOG(MSG_INFO, "*.exe found in the Content-Disposition (non-standard RFC)");
      req->info_server |= REQUEST_TYPE_MSDOS;
    }
  }
  else if( (line = is_header_type(df->data, "Transfer-Encoding:", &rest)) )  
  {
    token = line_get_next_tok(line, 0, NULL);
    if(!strcmp(token, "chunked")) 
    {
      set_request_body_type(req, __MSG_BODY_HAS_CHUNK);
      set_request_body_state(req, __MSG_BODY_CHUNK_SIZE_EXPECTED);
      D_LOG(MSG_DEBUG, "Transfer-Encoding: chunked");
      jfree(token);
      return 1;
    }
    jfree(token);
  }
  else if( (line = is_header_type(df->data, "Content-Length:", &rest)) )
  {
    token = line_get_next_tok(line, 0, NULL);
    req->total_len = atoi(token);

    jfree(token); 
    
    if( req->total_len > 0) // exclude Content-Length: 0   non-present body..
      set_request_body_type(req, __MSG_BODY_HAS_CONTENT_LENGTH);
    else
      req->flags |= __F_ZERO_BODY;

    D_LOG(MSG_DEBUG, "Content-Length: %d", req->total_len);
    return 1;
  }
  else if( (line = is_header_type(df->data, "Content-Type:", &rest)) )
  {
    req->flags  |= __F_MAY_HAVE_BODY; 
    if( !req->total_len )
      req->total_len = __UNKNOWN_LEN_MARKER;

    if(line_strmem_nocase(line, "msdos", &rest))
    {
      req->info_server |= REQUEST_TYPE_MSDOS;
    }
    /* TODO: get filter request .. and check also for Connection: Close  */
  }
  else if( is_empty_line(df->data, df->size)) // if end of header mark reqeust as body if needed
  {
    if( req->__body_type )
    {
      set_request_status(req, __MSG_CUR_BODY);
    }
    else if( req->flags & __F_MAY_HAVE_BODY && !(req->flags & __F_ZERO_BODY) )
    {
      set_request_status(req, __MSG_CUR_BODY);
      set_request_body_type(req, __MSG_BODY_ON_CLOSE); // for IIS support.. 
    } 
    else
      set_request_status(req, __MSG_COMPLETED);

    return 1;
  }

  return 0;
}


/* we must read all header until there is no more data to process it's client/server independent
 * NOTE: we must call this function until it return != PROTOCOL_AGAIN
 */

int process_header(proxy_session_t *session, df_t **new_df,   int peer)
{
  df_t *df=NULL; 
  LINE *line=NULL;
  size_t len;

  struct client_request *req = get_cur_request(session, peer);
  
  assert( session && new_df && (peer == PEER_CLIENT || peer == PEER_SERVER ) );
  assert( (peer == PEER_CLIENT && req->__status & __MSG_CUR_CLIENT ) || (peer == PEER_SERVER && req->__status & __MSG_CUR_SERVER) );
  assert( (req->__status & (__MSG_CUR_HEADER|__MSG_CUR_BODY)) == __MSG_CUR_HEADER );
   
  line = get_header_line(session,  &len, peer);

  *new_df = NULL;
  
  if( !line ) // no enought data available.. 
    return PROTOCOL_NO_DATA;

  df = alloc_df(line, len, DF_HEADER, 0);
  if( df == NULL )
    return PROTOCOL_ERROR;

  *new_df = df;
  
  if( is_empty_line(line, len) )
  {
    df->flags |= DF_F_EOF;
    return PROTOCOL_END;
  }

  return PROTOCOL_AGAIN;
}
