#include <stdio.h>
#include <sys/select.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>
#include "jsocket.h"
#include "jtypes.h"
#include "jselect.h"

#define DEFAULT_START_SET_SIZE 16


/* find  real_sock <-> jsocket binding (this will be done with direct mapped table..) */
static j_socket* find_jsock_from_set(j_socket_set_t* set, os_sock real_sock)
{
  size_t num = set->cur + 1,i;
  for(i=0; i< num; i++)
  {
    j_socket* jsock = set->j_socket_array[i];

    /* skip GAP */
    if(jsock == NULL)
      continue;

    if( real_sock == js_getsock(jsock) )
      return jsock;
  }

  return NULL;
}


/* NOTE: js_select() free it and return a new set
 *       Now Support Only READ ready socket handler
 */
int js_select(j_socket_set_t *set, j_socket_set_t **active_set, int sec)
{
  fd_set read_fd;
  os_sock max_fd = 0;
  FD_ZERO(&read_fd);
  size_t num = set->cur+1,i;
  struct timeval tv;
  int ret;
  j_socket *tmp;

  tv.tv_sec = sec;
  tv.tv_usec = 0;
  
  for(i=0; i<num; i++)
  {
    /* deleted gap ! */
    tmp = set->j_socket_array[i];
    if( !tmp )
      continue;

    os_sock real_sock = js_getsock(tmp);
    if(real_sock < 0)
    {
      /* BUG() */
      errno = EBADF;
      return -1;
    }

    FD_SET(real_sock, &read_fd);

    if(real_sock > max_fd)
      max_fd = real_sock;
  }
  
  ret = select(++max_fd, &read_fd, NULL, NULL, &tv);
  
  /* if some error occur or timer expire return immediatly */
  if(ret <= 0)
    return ret; 
 
  /* if some reader is ok... try to set in the j_socket_set_t the descriptor ready.. 
      now we use e busy loop, if it cause a  very huge load  i'll implement the whole as a hashing table */

  j_socket_set_t *new_set = get_j_socket_set(set->num);
  if( !new_set )  
  {
    errno = ENOMEM;
    return -1;
  }

 
  for(i=0; i<max_fd; i++)
  {
    if( FD_ISSET(i, &read_fd) )
    {
      j_socket *jsock = find_jsock_from_set(set, i);
      if( !jsock )
      {
        /* BUG() */
        errno = EINVAL;
        release_j_socket_set(new_set);
        return -1;
      } 

      if( !add_j_socket_to_set(new_set, jsock) )
      {
        errno = ENOMEM;
        release_j_socket_set(new_set);
        return -1;
      }
    }  
  }

  *active_set = new_set;
  
  return ret;
}
