#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <string.h>
#include "jtypes.h"
#include "jsocket.h"
#include "jmalloc.h"

/* op jsocket structure */

#define GET_PRIV(p) ((struct op_priv_data*)(p))

#define DEFAULT_BACKLOG 4

struct op_priv_data
{
  //struct sockaddr_in sock_name;
  os_sock sock;
  int flags;
};


/* op jsocket function implementation: useless forward declaration  */
j_socket* js_init_new_sock_tcp();
j_socket* js_init_sock_tcp(os_sock sock);
void js_destroy_sock_tcp(j_socket* sock);

static int op_js_read(void* p_data, char* buffer, size_t len) 
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return recv(priv->sock, buffer, len, 0); 
}

static int op_js_write(void* p_data, const char* c_buffer, size_t len) 
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return send(priv->sock, c_buffer, len, 0);
}

static int op_js_close(void* p_data)
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return close(priv->sock); 
}

static int op_js_connect(void* p_data, const struct sockaddr_in* peer, int opt)
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return connect(priv->sock, (struct sockaddr*)peer, sizeof(struct sockaddr_in)); 
}

static os_sock op_js_accept(void* p_data)
{
  size_t len = sizeof(struct sockaddr_in);
  struct op_priv_data* priv = GET_PRIV(p_data);
  return accept(priv->sock, NULL, &len);
}


static int op_js_listen(void* p_data)
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return listen(priv->sock, DEFAULT_BACKLOG);
}

static int op_js_bind(void* p_data, const struct sockaddr_in* peer)
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return bind(priv->sock, (struct sockaddr*)peer, sizeof(struct sockaddr_in));
}


static int op_js_getname(void* p_data, struct sockaddr_in* peer)
{
  size_t len = sizeof(struct sockaddr_in);
  struct op_priv_data* priv = GET_PRIV(p_data);
  return getpeername(priv->sock, (struct sockaddr*)peer, &len);
}

static int op_js_getsock(void* p_data)
{
  struct op_priv_data* priv = GET_PRIV(p_data);
  return priv->sock;
}


/* this take some flag */
static j_socket* get_jsock(os_sock sock)
{
  struct op_priv_data* p_data = jmalloc(sizeof(struct op_priv_data)); 
  
  if( !p_data )
    return NULL;
  
  j_socket* jsock = jmalloc(sizeof(j_socket));
  if( !jsock )
  {
    jfree(p_data);
    return NULL;
  }

  memset(p_data, 0x00, sizeof(struct op_priv_data));

  /* now init j_socket obj */
  jsock->p_data  = p_data;
  jsock->read    = &op_js_read;
  jsock->write   = &op_js_write;
  jsock->close   = &op_js_close;
  jsock->connect = &op_js_connect;
  jsock->accept  = &op_js_accept;
  jsock->bind    = &op_js_bind;
  jsock->listen  = &op_js_listen;
  jsock->getname = &op_js_getname;
  jsock->getsock = &op_js_getsock;

  /* set socket */
  p_data->sock = sock;
  
  return jsock;
}


j_socket* js_init_new_sock_tcp()
{
  os_sock sock = socket(PF_INET, SOCK_STREAM, 0); 
  if( sock == -1 )
    return NULL;
  
  return get_jsock(sock);
}

j_socket* js_init_sock_tcp(os_sock sock)
{
  return get_jsock(sock);
}

void js_destroy_sock_tcp(j_socket* sock)
{
  struct op_priv_data* priv = GET_PRIV(sock->p_data);

  /* close the socket if it is open.. */
  js_close(sock); 
  
  jfree(priv);
  jfree(sock); 
}

