/**
 **/
#include <string.h>
#include <stdio.h>

#include "resource.h"

/***
 *	ExploreResourceDataEntry
 ***/
BOOL ExploreResourceDataEntry(PEInfo *pInfo, PIMAGE_RESOURCE_DATA_ENTRY pDataEntry)
{
	if (pInfo == NULL || pDataEntry == NULL)
		return FALSE;

	return TRUE;
}

static BOOL IsPaddingX(char *ptr)
{
	BOOL bResult = FALSE;

	switch(*ptr) 
	{
		case 'P':
		case 'A':
		case 'D':
		case 'I':
		case 'N':
		case 'G':
		case 'X':
				bResult = TRUE;
			break;
		default:
			break;
	}

	return bResult;
}

static BOOL ExploreResourceSubDir(PEInfo *pInfo, PIMAGE_RESOURCE_DIRECTORY_ENTRY pHeader, PIMAGE_RESOURCE_DIRECTORY pResDirectory, PIMAGE_RESOURCE_DIRECTORY pBase)
{
	PIMAGE_RESOURCE_DIRECTORY_ENTRY pDirEntry =
		(PIMAGE_RESOURCE_DIRECTORY_ENTRY)((DWORD)(pResDirectory) + 0x10);

	int entries = pResDirectory->NumberOfIdEntries + pResDirectory->NumberOfNamedEntries;

	for(int i=0; i < entries; i++) {
		if (pDirEntry[i].DataIsDirectory)
			ExploreResourceSubDir(pInfo, &pDirEntry[i], (PIMAGE_RESOURCE_DIRECTORY)(pDirEntry[i].OffsetToDirectory + (DWORD)pBase), pBase);
		else {
			// MANIFEST Addr
			PIMAGE_RESOURCE_DATA_ENTRY pDataEntry = 
				(PIMAGE_RESOURCE_DATA_ENTRY)(pDirEntry[i].OffsetToData + (DWORD) pBase);

#ifdef _CONSOLE
			printf("\nFound Data at: %x:Size %x", 
				pDataEntry->OffsetToData,
				pDataEntry->Size);
#endif

			char *ptr = (char*)(RvaToOffset(pDataEntry->OffsetToData, pInfo->ImageNtHeaders) 
				+ (DWORD)pInfo->FileBase);

#ifdef _CONSOLE
			printf("\nManifest %s", ptr);
#endif

			DWORD extraSpace = 0;

			while(IsPaddingX(&ptr[pDataEntry->Size+extraSpace]))
				extraSpace += 1;

#ifdef _CONSOLE
			printf("\nExtra space to expand manifest: %d", extraSpace);
#endif

			char *manifestPtr = new char[pDataEntry->Size+1];
			memset(manifestPtr, 0, pDataEntry->Size+1);
			memcpy(manifestPtr, ptr, pDataEntry->Size);

			char *beginTag = "<requestedPrivileges>";
			char *endTag = "</requestedPrivileges>";

			char *ptrRequestedPrivileges = strstr(manifestPtr, beginTag);
			char *endRequestedPrivileges = strstr(manifestPtr, endTag);
			char *newManifest = new char[pDataEntry->Size+100];

			if (ptrRequestedPrivileges != NULL) {
				memset(newManifest, 0, pDataEntry->Size+100);

				strncpy(newManifest, manifestPtr, ptrRequestedPrivileges - manifestPtr);
				strcat(newManifest, beginTag);
				// embed manifest
				strcat(newManifest, "<requestedExecutionLevel level=\"requireAdministrator\" uiAccess=\"false\"/>");
				strcat(newManifest, endRequestedPrivileges);
			}

			if (strlen(newManifest) > (pDataEntry->Size + extraSpace)) {
				delete newManifest;
				delete manifestPtr;
#ifdef _CONSOLE
				printf("\nUnable to change the manifest");
#endif
				return FALSE;
			}

			pDataEntry->Size = strlen(newManifest);	// Change current size of manifest
			memcpy(ptr, newManifest, pDataEntry->Size);	// Change manifest!

			//TiXmlDocument manifest(manifestPtr);

			//manifest.
			delete manifestPtr;
			delete newManifest;
			return TRUE;		// manifest!
		}
		return TRUE;
	}

	return FALSE;
}

static BOOL ExploreResourceDir(PEInfo *pInfo, PIMAGE_RESOURCE_DIRECTORY pResDirectory)
{
	PIMAGE_RESOURCE_DIRECTORY_ENTRY pDirEntry =
		(PIMAGE_RESOURCE_DIRECTORY_ENTRY)((DWORD)(pResDirectory) + 0x10);

	int entries = pResDirectory->NumberOfIdEntries + pResDirectory->NumberOfNamedEntries;

	for(int i=0; i < entries; i++) {
		if (pDirEntry[i].DataIsDirectory)
			ExploreResourceSubDir(pInfo, &pDirEntry[i], (PIMAGE_RESOURCE_DIRECTORY)(pDirEntry[i].OffsetToDirectory + (DWORD)pResDirectory),
			pResDirectory);
	}

	return FALSE;
}

/***
 *	Explorer a resource Section
 ***/
static BOOL ExploreResourceSection(PEInfo *pInfo, PIMAGE_SECTION_HEADER pRsrc)
{
	if (pInfo == NULL || pRsrc == NULL)
		return FALSE;

	PIMAGE_RESOURCE_DIRECTORY pResDirectory = (PIMAGE_RESOURCE_DIRECTORY)RvaToOffset(pRsrc->VirtualAddress, pInfo->ImageNtHeaders);

	pResDirectory = (PIMAGE_RESOURCE_DIRECTORY)((DWORD)pResDirectory + (DWORD)pInfo->FileBase);

	return ExploreResourceDir(pInfo, pResDirectory);
}
/***
 *	ChangeManifest
 *	Try to identify the resource with manifest, and to change adding privilege for UAC
 ***/
BOOL ChangeManifest(PEInfo *pInfo)
{
	if (pInfo == NULL)
	{
		return FALSE;
	}

	PIMAGE_SECTION_HEADER pSection = IMAGE_FIRST_SECTION(pInfo->ImageNtHeaders);

	for(int i=0; i < pInfo->ImageFileHeader->NumberOfSections; i++) {
		if (strcmp((char*)pSection[i].Name, ".rsrc") == 0) {
			// Explore section
			ExploreResourceSection(pInfo, &pSection[i]);
		}
	}

	return TRUE;
}
